// Module implemented by Olivier Morin (08-2014) on Baptiste's code basis
// used for acquisition of an histogram from APD pulsed signal.

// Data from in, in_2 and in_3 is read from address 0xF, 0xF+1 and 0xF+2
// from address=0b10100=0x14=20 the number of triggers received can be read (trigg_cnt)

module APDacq_main(
    input wire clk, //clock from FPGA board
	input wire trig,// Trig in to start acquisition
	input wire in_1,// signal from first APD
	input wire in_2,
	input wire in_3,
	
	// FX2 (USB) interface -----------------------------------------------------------------------------
		input  wire      fx2Clk_in,     // 48MHz clock from FX2
		output wire[1:0] fx2Addr_out,   // select FIFO: "10" for EP6OUT, "11" for EP8IN
		inout  wire[7:0] fx2Data_io,    // 8-bit data to/from FX2

		// When EP6OUT selected: -> PC to FPGA
		output wire      fx2Read_out,   // asserted (active-low) when reading from FX2
		output wire      fx2OE_out,     // asserted (active-low) to tell FX2 to drive bus
		input  wire      fx2GotData_in, // asserted (active-high) when FX2 has data for us

		// When EP8IN selected: -> FPGA to PC
		output wire      fx2Write_out,  // asserted (active-low ) when writing to FX2
		input  wire      fx2GotRoom_in, // asserted (active-high) when FX2 has room for more data from us
		output wire      fx2PktEnd_out  // asserted (active-low ) when a host read needs to be committed early	
    );
		
	//reg [31:0] mem [6249:0];
	//reg [31:0] mem2 [6249:0];
	reg [31:0] mem [3124:0];    // Memory to store data from in_1 for each acquisition sequence (each bit is a clock cycle time bin, data is loaded from buffer every 32 bits
	reg [31:0] mem2 [3124:0];   // idem for in_2
	reg [31:0] mem3 [3124:0];   // idem for in_3

	reg running;		// for trigger, is the sequence acquisition running?
	reg trig_is_low;	// for trigger, used for trig edge detection
	
	// Channel read/write interface -----------------------------------------------------------------
		wire[6:0]  chanAddr;  // the selected channel (0-127)

		// Host >> FPGA pipe:
		wire[7:0]  h2fData;   // data lines used when the host writes to a channel
		wire       h2fValid;  // '1' means "on the next clock rising edge, please accept the data on h2fData_out"
		wire       h2fReady;  // channel logic can drive this low to say "I'm not ready for more data yet"

		// Host << FPGA pipe:
		wire[7:0]  f2hData;   // data lines used when the host reads from a channel
		wire       f2hValid;  // channel logic can drive this low to say "I don't have data ready for you"
		wire       f2hReady;  // '1' means "on the next clock rising edge, put your next byte of data on f2hData_in"
		// ----------------------------------------------------------------------------------------------

		// Needed so that the comm_fpga_fx2 module can drive both fx2Read_out and fx2OE_out
		wire       fx2Read;

		// Registers implementing the channels
		reg[7:0]   int_mutex          = 8'h00;	//mutex is written by PC host, acquisition can only be triggered when int_mutex==mutex
		reg[7:0]   mutex              = 8'h00; 	//thus data in fpga memory cannot change while being transferred to PC
		wire[7:0]  mutex_next;
		
		reg[7:0] trigg_cnt = 0;		// used to keep track of the number of acquisition triggers received, readable form address 10100 of fx2 USB communication
		
		// For inputs to memory transfer
		reg[31:0] buff_32 = 0;      // buffer used to temporarily save data from in1, each bit <-> clock cycle time bin
		reg[31:0] buff_32_2 = 0;    // idem for in2
		reg[31:0] buff_32_3 = 0;    // idem for in3
		reg[16:0] mem_cnt = 0;      // data is transfered into mem (mem2 or mem3) when buffer is full, then mem_cnt is incremented to store in next memory 32bits register
		reg[4:0] cnt_32 =0;         // 5bits clock cycle counter used to address individual bits of buff32 (back to 0 when buff32 is full)
		
		// For memory -> PC (via USB transfer)
		reg[31:0] buff_4 = 0;       // buffer used to transfer data from mem to PC (via USB)
		reg[31:0] buff_4_2 = 0;     // idem for mem2
		reg[31:0] buff_4_3 = 0;     // idem for mem3
		reg[16:0] dat_cnt = 0;      // used when transfering data from mem to buff_4 for USB reading, when buff_4 is full, dat_cnt is incremented to read from next memory 32bits register
		reg[1:0] cnt_4 =0;  // 8bits are transfered at once, con_4 allows to split data transfer from buff_4 in 4 fx2clock cycles

		// Infer registers
		always @(posedge fx2Clk_in)
		begin
			if(f2hReady && (chanAddr == 7'b0001111 || chanAddr == 7'b0010000 || chanAddr == 7'b0010001)) //  chaque fois qu'il y a des data  lire
			begin
				if (cnt_4==3 && dat_cnt<3124)
				begin
					dat_cnt = dat_cnt+2'b01;
				end
			
				else if (cnt_4==3 && dat_cnt>=3124) // on a finie de lire les data on rinitialise
				begin
					dat_cnt=0;
				end
				cnt_4 = cnt_4+2'b01;
			end
						
			buff_4[31:0] = mem[dat_cnt];
			buff_4_2[31:0] = mem2[dat_cnt];
			buff_4_3[31:0] = mem3[dat_cnt];
						
			mutex <= mutex_next;			
		end
		assign mutex_next =
			(chanAddr == 7'b0010011 && h2fValid == 1'b1) ? h2fData : mutex;
		
	// Select values to return for each channel when the host is reading
	assign f2hData =
		(chanAddr == 7'b0001111 && dat_cnt<3125) ? buff_4[cnt_4*8+:8] :
		(chanAddr == 7'b0010000 && dat_cnt<3125) ? buff_4_2[cnt_4*8+:8] :
		(chanAddr == 7'b0010001 && dat_cnt<3125) ? buff_4_3[cnt_4*8+:8] :
		(chanAddr == 7'b0010010) ? int_mutex :
		(chanAddr == 7'b0010011) ? mutex :
		(chanAddr == 7'b0010100) ? trigg_cnt :
		
		8'h00;
		
	// Assert that there's always data for reading, and always room for writing
	assign f2hValid = 1'b1;
	assign h2fReady = 1'b1;                                                  //END_SNIPPET(registers)

	// CommFPGA module
	assign fx2Read_out = fx2Read;
	assign fx2OE_out = fx2Read;
	assign fx2Addr_out[1] = 1'b1;  // Use EP6OUT/EP8IN, not EP2OUT/EP4IN.
	comm_fpga_fx2 comm_fpga_fx2(
		// FX2 interface
		.fx2Clk_in(fx2Clk_in),
		.fx2FifoSel_out(fx2Addr_out[0]),
		.fx2Data_io(fx2Data_io),
		.fx2Read_out(fx2Read),
		.fx2GotData_in(fx2GotData_in),
		.fx2Write_out(fx2Write_out),
		.fx2GotRoom_in(fx2GotRoom_in),
		.fx2PktEnd_out(fx2PktEnd_out),

		// Channel read/write interface
		.chanAddr_out(chanAddr),
		.h2fData_out(h2fData),
		.h2fValid_out(h2fValid),
		.h2fReady_in(h2fReady),
		.f2hData_in(f2hData),
		.f2hValid_in(f2hValid),
		.f2hReady_out(f2hReady)
	);
	
    always@(posedge clk)
    begin
		if(1)// (if obsolte a priori
		begin
			if(trig==0) // make sure trig is low before detecting a trig posedge
			begin
				trig_is_low=1;
			end
				
			if(trig_is_low & trig)//edge_detected
			begin
				trigg_cnt = trigg_cnt+1'b1;
				if(int_mutex[0]==mutex[0])// start a new acquisition from APDs (as requested)
					running=1; 
			end
			if (trig==1)
				trig_is_low=0;
		end
				
		if (running==1) // si l'acquisition from APDs est dclanche
		begin	
			buff_32[cnt_32]=in_1;
			buff_32_2[cnt_32]=in_2;
			buff_32_3[cnt_32]=in_3;
				
			
			if(cnt_32==31) // le buffer 32 bit est plein (!= indique que l'on est pas en dbut de squence
			begin
				mem[mem_cnt]=buff_32[31:0];  // on enregistre les 32 bits
				mem2[mem_cnt]=buff_32_2[31:0];  // on enregistre les 32 bits
				mem3[mem_cnt]=buff_32_3[31:0];  // on enregistre les 32 bits
				mem_cnt = mem_cnt + 1; // on passe  la mmoire suivante
				//buff_32=0;
			end
			
			cnt_32 = cnt_32 + 1; // on passe au bit suivant
			
			if (mem_cnt>=3125) // on a rempli l'intgralit de la mmoire -> on stop l'acquisition
			begin
				running=0;
				mem_cnt=0;
				int_mutex[0]=int_mutex[0]+1;
			end
		end
    end

endmodule
