''' Code for acquisition for APD when running the nanofiber experiment in DLCZ configuration.
'''
#Note: a SEGMENT coresponds to a single write/read pulse, there are N_seq segments in a waveform acquired (one MOT cycle) and N_seg segments in a set of waveforms.

# TODO and NOTES saved from Olivier Morin's Version
# Recoded by Baptiste Gouraud on 2015-01-07
#======

# ajouter le armed single de l'oscillo
# cas o il n'y a pas de trigg
# probleme qd fichier config n'existe pas
# revoir la gestion de treshold
# duplicate problem (>2) to fix
# gerer le fait que les deux APD n'ait pas la mme amplitude
# et l'oscillo pas rgler sur le mme gain

####   NOTES  ####

# l'IP de loscillo utilis se trouve dans setup.py

# the problem of the rescaling has been fixed by using re_init_plot.
# it was due to the axvline... however, its behaviour is not really clear
# and probably the found solution is not very "clean" (i.e. memory leaks?)

# the Run function doesn't block the UI thanks to the wx.Yield() function
# however the responsivness is not wonderful and the other strategy would
# be to use threads see http://wiki.wxpython.org/LongRunningTasks

# when using the sequence mode of the scope the exact number of data
# point can vary for instance at 4ns resolution for 1ms we have 250002 points
# for this reason when one reads the second sequence it should be shift exactly
# of this amount otherwise a shift is accumulate
# or the other solution is to manage that with the information contain in the wave desc

##################
import ctypes
from ctypes import *

import os
import pprint
import random
import sys
try:
    import wx
    import wx.py.crust as pyshell
except ImportError:
	raise ImportError,"The wxPython module is necessary to run this program."

import time
from datetime import datetime

#import FPGA acquisition interface
import APDacq as fpga_acq

'''
import Lecrunch
from Lecrunch import sock
from Lecrunch import lecroy
from Lecrunch import setup
'''
import ConfigParser

import matplotlib
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import \
    FigureCanvasWxAgg as FigCanvas, \
    NavigationToolbar2WxAgg as NavigationToolbar
import numpy as np
import pylab

import random

# generate random data that emulate the APD signal recorded by the oscilloscope
def wfGen(res,t1,period,choice):
    ''' Generate a waveform similar to that of an oscilloscope for simulation.
        Parameters: time resolution (ns), period = time length of a segment (ns), choice: ref,abs,etc, t1 time (ns) of the pulse in a segment '''
    N_seq = 1000
    seg_size = period/res
    elec_pulse_size = 15    #pulse amplitude (V?)
    light_pulse_size = 160  #pulse length 
    wf1 = np.array([])
    if choice==0:
        maxx=18
    elif choice==1:
        maxx=0.2
    elif choice==2:
        maxx=1
    elif choice ==3:
        maxx=15
    else:
        maxx=5
    
    for i in range(0,N_seq):
        seg1 = np.zeros(shape=(seg_size),dtype=np.int)
        seg2 = np.zeros(shape=(seg_size),dtype=np.int)
        dt = np.ceil(random.uniform(0,light_pulse_size/res))
        if random.uniform(-9,maxx)>0:
            seg1[t1/res+dt:t1/res+dt+elec_pulse_size/res]=10

        #if random.uniform(-1,2)>0:
        #    seg1[200/res+dt:200/res+dt+elec_pulse_size/res]=10

        if 100<i<300 and random.uniform(-1,2)>0: #deuniformise distribution
            seg1=np.zeros(len(seg1))

        wf1 = np.append(wf1,seg1)

    return wf1
     
class DLCZ_nano_fiber(wx.Frame):
    """ The main frame of the application
    """
    title = 'DLCZ with nanofiber'
    
    def __init__(self,acq_mode,exp_ctrl=False):
        wx.Frame.__init__(self, None, -1, self.title)
        self.exp_ctrl=exp_ctrl
        if exp_ctrl: #take the experiment control, select the proper instrument interfaces and load the proper timing config
            sys.path.append(os.path.abspath(os.path.dirname(__file__) + '/' + '../..'))
            from chaudFroid import ChaudFroid
            self.chaudFroid=ChaudFroid(waveplate=0,aramis_phaselock=0)
            self.chaudFroid.synchronizer.loadfromfile('configs/2014-09-18_pulseshaping.cfg')

        self.N_seq=0

        self.acq_mode=acq_mode
        self.Run = False    # is the acquisition running?
        
        self.save_dir = "C:\\Users\\TirageDeFibres\\Data\\Nano_Fiber_DLCZ\\"

        self.color_dic={}
        self.color_dic["APD_1"]=(1,0,0,1)
        self.color_dic["APD_2"]=(1,0,1,1)
        self.color_dic["APD_3"]=(0,1,0,1)
        self.color_dic["APD_2_cond"]=(1,0,1,1)
        self.color_dic["APD_3_cond"]=(0,1,0,1)
        
        self.res = 1 # time resolution in nanoseconds
        self.period = 1000 # period of the read/write cycle in nanoseconds
        self.seg_size = self.period/self.res # number of point in a segment

        #### Create histograms arrays ####
        # time cursors for the two histogram, ta and tb, 1 for reference, 2 for memory readout, back from control contamination
        self.t1a = 90.0
        self.t1b = 160.0
        self.t2a = 300.0
        self.t2b = 380.0
        self.ta_back = 310.0
        self.tb_back = 360.0

        # initialization of hitograms data array
        #self.hist_X = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_APD_1 = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_APD_2 = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_APD_3 = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_APD_2_cond = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_APD_3_cond = (np.zeros((self.seg_size,), dtype=np.float))

        #self.hist_Y1 = (np.zeros((self.seg_size,), dtype=np.int)) # intermediate arrays for APD1, 2 and 3
        #self.hist_Y2 = (np.zeros((self.seg_size,), dtype=np.int))
        #self.hist_Y3 = (np.zeros((self.seg_size,), dtype=np.int))

        self.hist_APD_1 = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_APD_2 = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_APD_3 = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_APD_2_cond = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_APD_3_cond = (np.zeros((self.seg_size,), dtype=np.int))

        self.x_vs_pulses = (np.zeros((self.seg_size,), dtype=np.float))
        self.y_vs_pulses = (np.zeros((self.seg_size,), dtype=np.float))

        #################################
        
        self.create_menu()
        self.create_main_panel()
        
        self.Load_config()        

        if self.acq_mode=="lecroy":
            self.scope=lecroy.LeCroyScope(setup.scope_ip)

        elif self.acq_mode=="fpga":
            self.Fpga_Acq = fpga_acq.FPGA_acq()

    #### Create widgets utils. ####
    def create_menu(self):
        self.menubar = wx.MenuBar()
        
        menu_file = wx.Menu()
        m_expt = menu_file.Append(-1, "&Save plot\tCtrl-S", "Save plot to file")
        self.Bind(wx.EVT_MENU, self.on_save_plot, m_expt)
        m_expt2 = menu_file.Append(-1, "&Save config.", "Save config.")
        self.Bind(wx.EVT_MENU, self.on_save_cfg, m_expt2)
        m_expt3 = menu_file.Append(-1, "&Load config.", "Load config.")
        self.Bind(wx.EVT_MENU, self.on_load_cfg, m_expt3)
        menu_file.AppendSeparator()
        m_exit = menu_file.Append(-1, "E&xit\tCtrl-X", "Exit")
        self.Bind(wx.EVT_MENU, self.on_exit, m_exit)
        self.Bind(wx.EVT_CLOSE, self.on_exit)
                
        self.menubar.Append(menu_file, "&File")
        self.SetMenuBar(self.menubar)

    def create_main_panel(self):
        self.panel = wx.Panel(self)
                
        ##  BOX FOR INPUT PARAMETERS  ##
        
        self.param_box_text = wx.StaticBox(self.panel, -1, "Parameters")
        self.param_box = wx.StaticBoxSizer(self.param_box_text, wx.VERTICAL)

        self.period_box = wx.BoxSizer(wx.HORIZONTAL)
        self.text_period = wx.StaticText(self.panel, -1,label="period read/write (ns) = ")
        self.spbutton_period = wx.SpinCtrlDouble(self.panel, value=str(self.period), size=(70,-1),
                                         min=0.0, max=1000000, inc=100)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_period)
        self.period_box.Add(self.text_period, 0, wx.ALL, 0)
        self.period_box.Add(self.spbutton_period, 0, wx.ALL, 0)

        #### t1 ####
        self.time1_box = wx.BoxSizer(wx.HORIZONTAL)

        self.text_t1a = wx.StaticText(self.panel, -1,label="t1a (ns) = ")
        self.spbutton_t1a = wx.SpinCtrlDouble(self.panel, value=str(self.t1a), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t1a)
        self.text_t1b = wx.StaticText(self.panel, -1,label="t1b (ns) = ")
        self.spbutton_t1b = wx.SpinCtrlDouble(self.panel, value=str(self.t1b), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t1b)

        self.time1_box.Add(self.text_t1a, 0, wx.ALL, 0)
        self.time1_box.Add(self.spbutton_t1a, 0, wx.ALL, 0)
        self.time1_box.AddSpacer(20)
        self.time1_box.Add(self.text_t1b, 0, wx.ALL, 0)
        self.time1_box.Add(self.spbutton_t1b, 0, wx.ALL, 0)

        #### t2 ####
        self.time2_box = wx.BoxSizer(wx.HORIZONTAL)

        self.text_t2a = wx.StaticText(self.panel, -1,label="t2a (ns) = ")
        self.spbutton_t2a = wx.SpinCtrlDouble(self.panel, value=str(self.t2a), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t2a)
        self.text_t2b = wx.StaticText(self.panel, -1,label="t2b mem (ns) = ")
        self.spbutton_t2b = wx.SpinCtrlDouble(self.panel, value=str(self.t2b), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t2b)

        self.time2_box.Add(self.text_t2a, 0, wx.ALL, 0)
        self.time2_box.Add(self.spbutton_t2a, 0, wx.ALL, 0)
        self.time2_box.AddSpacer(20)
        self.time2_box.Add(self.text_t2b, 0, wx.ALL, 0)
        self.time2_box.Add(self.spbutton_t2b, 0, wx.ALL, 0)

        ####
        self.param_box.Add(self.period_box, 0, wx.ALL, 5)
        self.param_box.Add(self.time1_box, 0, wx.ALL, 5)
        self.param_box.Add(self.time2_box, 0, wx.ALL, 5)

        ## BOX FOR OUTPUT RESULTS (basic data analysis)##

        self.N_seg_APD_1=0
        self.N_seg_APD_2=0
        self.N_seg_APD_3=0
        self.N_seg_APD_2_cond=0
        self.N_seg_APD_3_cond=0

        self.n_APD_1=0.0
        self.n_APD_2=0.0
        self.n_APD_3=0.0
        self.n_APD_2_cond=0.0
        self.n_APD_3_cond=0.0

        self.N_APD_1=0
        self.N_APD_2=0
        self.N_APD_3=0
        self.N_APD_2_cond=0
        self.N_APD_3_cond=0

        self.P1 = -1.0
        self.P2 = -1.0
        self.P12 = -1.0
        self.PC = -1.0
        self.G12 = -1.0
        self.N123 = 0.0
        self.N23 = 0.0
        self.G2 = -1.0
        self.G2_noselect = -1.0
       
        self.N_seg = 0
        
        self.results_box_text = wx.StaticBox(self.panel, -1, "Results")
        self.results_box = wx.StaticBoxSizer(self.results_box_text, wx.VERTICAL)
        
        self.grid_results = wx.FlexGridSizer( cols=2 )

        self.text_n_APD_1 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n1 = "+str(self.n_APD_1))
        self.text_n_APD_1.SetForegroundColour(tuple(i * 255 for i in self.color_dic["APD_1"]))
        self.text_N_APD_1 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="        ("+str(self.N_APD_1)+"/"+str(self.N_seg_APD_1)+")")

        self.text_n_APD_1.SetToolTip(wx.ToolTip("n_ : mean number of photon between the cursors\n"
                                                "N_ : number of detection between the cursors\n"
                                                "N_seg_ : number of trials\n"
                                                "n_ =   (N_/N_seg)"))
        
        self.text_n_APD_2 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n2 = "+str(self.n_APD_2))
        self.text_n_APD_2.SetForegroundColour(tuple(i * 255 for i in self.color_dic["APD_2"]))
        self.text_N_APD_2 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_APD_2)+"/"+str(self.N_seg_APD_2)+")")
        
        self.text_n_APD_2_cond = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n2_cond = "+str(self.n_APD_2_cond))
        self.text_n_APD_2_cond.SetForegroundColour(tuple(i * 255 for i in self.color_dic["APD_2_cond"]))
        self.text_N_APD_2_cond = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_APD_2_cond)+"/"+str(self.N_seg_APD_2_cond)+")")
        
        self.text_n_APD_2.SetToolTip(wx.ToolTip("n_ : mean number of photon between the cursors\n"
                                                "N_ : number of detection between the cursors\n"
                                                "N_seg_ : number of trials\n"
                                                "n_ =   (N_/N_seg)"))

        self.text_n_APD_3 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n3 = "+str(self.n_APD_3))
        self.text_n_APD_3.SetForegroundColour(tuple(i * 255 for i in self.color_dic["APD_3"]))
        self.text_N_APD_3 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_APD_3)+"/"+str(self.N_seg_APD_3)+")")

        self.text_n_APD_3_cond = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n3_cond = "+str(self.n_APD_3_cond))
        self.text_n_APD_3_cond.SetForegroundColour(tuple(i * 255 for i in self.color_dic["APD_3_cond"]))

        self.text_N_APD_3_cond = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="        ("+str(self.N_APD_3_cond)+"/"+str(self.N_seg_APD_3_cond)+")")

        self.text_n_APD_3.SetToolTip(wx.ToolTip("n_ : mean number of photon between the cursors\n"
                                                "N_ : number of detection between the cursors\n"
                                                "N_seg_ : number of trials\n"
                                                "n_ =   (N_/N_seg)"))

        self.grid_results.Add( self.text_n_APD_1,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_APD_1,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_APD_2,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_APD_2,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_APD_3,  0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_APD_3,  0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_APD_2_cond, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_APD_2_cond, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_APD_3_cond,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_APD_3_cond,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )

        self.results_box.Add( self.grid_results, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )

        self.text_P1 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="P1 = "+"%.2f" % self.P1)
        self.text_P1.SetToolTip(wx.ToolTip("P1 = P1 ;-)"))
        self.results_box.Add( self.text_P1, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )


        self.text_P2 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="P2 = "+"%.2f" % self.P2)
        self.text_P2.SetToolTip(wx.ToolTip("P2 = n2+n3"))
        self.results_box.Add( self.text_P2, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )

        self.text_P12 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="P12 = "+"%.2f" % self.P12)
        self.text_P12.SetToolTip(wx.ToolTip("P12 = n2_cond + n3_cond = P(1 and 2)"))
        self.results_box.Add( self.text_P12, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )


        self.text_PC = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="PC = "+"%.2f" % self.PC)
        self.text_PC.SetToolTip(wx.ToolTip("PC = P12/P1 = P(2|1)"))
        self.results_box.Add( self.text_PC, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )


        self.text_G12 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="G12 = "+"%.2f" % self.G12)
        self.text_G12.SetToolTip(wx.ToolTip("G12 = P12/(P1*P2)  >2 if non-classical \n =4/G2"))
        self.results_box.Add( self.text_G12, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )


        self.text_G2 = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="G2 = "+"%.2f" % self.G2+"      N123 = %d" % self.N123)
        self.text_G2.SetToolTip(wx.ToolTip("G2=w= P1*P123/(n2_cond*n3_cond) = P(2and3 |1)/(P(2|1)*P(3|1))  \n<1 if nonclassical, 0 for single photon"))
        self.results_box.Add( self.text_G2, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )


        self.text_G2_noselect = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="G2n = "+"%.2f" % self.G2_noselect+"      N23 = %d" % self.N23)
        self.text_G2_noselect.SetToolTip(wx.ToolTip("G2n = P(2and3)/(n2*n3) \nG2, not conditioned by 1"))
        self.results_box.Add( self.text_G2_noselect, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )


        ## BOX FOR RUN ##
        
        self.run_box_text = wx.StaticBox(self.panel, -1, "Run")
        self.run_box = wx.StaticBoxSizer(self.run_box_text, wx.HORIZONTAL)
        #self.sampleList = ['APD_1erence', 'APD_3ground', 'APD_2orption', 'APD_2_condparency','APD_3_condory']
        #self.run_choice = wx.RadioBox(
        #        self.panel, -1, "", wx.DefaultPosition, wx.DefaultSize,
        #        self.sampleList, 1, wx.RA_SPECIFY_COLS | wx.NO_BORDER
        #        )
        
        #self.run_choice.SetToolTip(wx.ToolTip("choose the type of data that gonna be measured"))
  
        self.run_button = wx.ToggleButton(self.panel, -1, "Run/Stop")
        self.run_button.SetValue(False)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.on_Run_button, self.run_button)

        #self.ctrl_exp_checkbx = wx.CheckBox(self.panel, -1, "Auto Control")
        #self.text_cycles = wx.StaticText(self.panel, -1,label="Acq cycles: ")
        #self.spCtrl_cycles = wx.SpinCtrl(self.panel, initial=100,value=str(100), size=(70,-1),
        #                                 min=0, max=1000000000)

        #self.run_options_box = wx.BoxSizer(wx.VERTICAL)
        #self.cycles_box = wx.BoxSizer(wx.HORIZONTAL)
        
        #self.cycles_box.Add(self.text_cycles, 0, wx.ALL, 0)
        #self.cycles_box.Add(self.spCtrl_cycles, 0, wx.ALL, 0)
        #self.run_options_box.Add(self.ctrl_exp_checkbx, 0, wx.ALL, 0)
        #self.run_options_box.AddSpacer(10)
        #self.run_options_box.Add(self.cycles_box, 0, wx.ALL, 0)
        
        
        #self.run_box.Add(self.run_choice, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, 1)
        #self.run_box.AddSpacer(10)
        self.run_box.Add(self.run_button, 0, wx.EXPAND|wx.RIGHT|wx.BOTTOM, 3)
        self.run_box.AddSpacer(10)
        #self.run_box.Add(self.run_options_box, 0, wx.EXPAND|wx.RIGHT|wx.BOTTOM, 3)
        
        ## BOX FOR SAVING TASKS ##

        self.save_box_text = wx.StaticBox(self.panel, -1, "Save")
        self.save_box = wx.StaticBoxSizer(self.save_box_text, wx.VERTICAL)
        self.save_checkbox = wx.BoxSizer(wx.HORIZONTAL)
    
        self.save_wf_check = wx.CheckBox(self.panel, -1, " waveforms")
        self.save_plot_check = wx.CheckBox(self.panel, -1, " plot")
        self.save_trigger_check = wx.CheckBox(self.panel, -1, " triggers")
        self.save_report_check = wx.CheckBox(self.panel, -1, " report")
        self.comments_stat = wx.StaticText(self.panel, -1, "Comments")
        self.comments_ctrl = wx.TextCtrl(self.panel, -1,
                        "",
                       size=(300,100), style=wx.TE_MULTILINE|wx.TE_PROCESS_ENTER)
        self.save_dir_bt = wx.Button(self.panel,50,"save dir.")#
        self.Bind(wx.EVT_BUTTON, self.on_save_dir, self.save_dir_bt)
        
        self.save_checkbox.Add(self.save_wf_check, 0, wx.ALL, 0)
        self.save_checkbox.AddSpacer(4)
        self.save_checkbox.Add(self.save_plot_check, 0, wx.ALL, 0)
        self.save_checkbox.AddSpacer(4)
        self.save_checkbox.Add(self.save_trigger_check, 0, wx.ALL, 0)
        self.save_checkbox.AddSpacer(4)
        self.save_checkbox.Add(self.save_report_check, 0, wx.ALL, 0)
        self.save_box.Add(self.save_checkbox, 0, wx.ALL, 0)
        self.save_box.AddSpacer(4)
        self.save_box.Add(self.comments_stat, 0, wx.ALL, 0)
        self.save_box.Add(self.comments_ctrl, 0, wx.ALL, 0)
        self.save_box.Add(self.save_dir_bt, 0, wx.ALL, 0)
  
        ## PLOT DISPLAY BOX ##
        self.dpi = 100        
        self.fig, (self.axes1,self.axes2,self.axes3) = pylab.subplots(3, sharex=True)
        self.fig_vs_pulses, (self.ax_vs_pulses) = pylab.subplots(1, sharex=True)
        self.fig_coherence, (self.ax_coherence) = pylab.subplots(1, sharex=True)
        self.init_plot()
        #self.init_plot_vs_pulse()
        
        self.nb = wx.Notebook(self.panel)

        self.canvas = FigCanvas(self.nb, -1, self.fig)
        self.canvas_vs_pulses = FigCanvas(self.nb, -1, self.fig_vs_pulses)
        self.canvas_coherence = FigCanvas(self.nb, -1, self.fig_coherence)
        
        self.hbox1 = wx.BoxSizer(wx.HORIZONTAL)

        self.hbox2 = wx.BoxSizer(wx.VERTICAL)

        self.hbox2.Add(self.param_box, border=5, flag=wx.ALL)
        self.hbox2.AddSpacer(4)
        
        self.hbox2.Add(self.results_box, border=5, flag=wx.ALL)
        self.hbox2.AddSpacer(4)
        
        self.hbox2.Add(self.save_box, border=5, flag=wx.ALL)

        self.hbox2.AddSpacer(4)
        
        self.hbox1.Add(self.run_box, border=5, flag=wx.ALL)
    
        # add the pages to the notebook with the label to show on the tab
        self.nb.AddPage(self.canvas, "Main")
        self.nb.AddPage(self.canvas_vs_pulses, "vs pulse")
        self.nb.AddPage(self.canvas_coherence, "Coherence time")

        
        self.vbox = wx.BoxSizer(wx.VERTICAL)
        self.vbox.Add(self.nb, 0, flag=wx.LEFT | wx.TOP | wx.GROW)        
        self.vbox.Add(self.hbox1, 0, flag=wx.ALIGN_LEFT | wx.TOP)

        self.mainbox = wx.BoxSizer(wx.HORIZONTAL)
        self.mainbox.Add(self.vbox, 2, flag=wx.LEFT | wx.TOP | wx.GROW)        
        self.mainbox.Add(self.hbox2, 1, flag=wx.ALL | wx.ALIGN_CENTER_HORIZONTAL)        
        
        
        self.panel.SetSizer(self.mainbox)
        self.vbox.Fit(self)
        self.mainbox.Fit(self)

    #### Tools for plotting. ####
    def init_plot(self):
        self.axes1.cla()
        self.axes2.cla()
        self.axes3.cla()
        
        self.ax_vs_pulses.cla()
        
        self.axes1.set_axis_bgcolor('black')
        self.axes2.set_axis_bgcolor('black')
        self.axes3.set_axis_bgcolor('black')
        self.ax_vs_pulses.set_axis_bgcolor('black')

        self.axes1.set_xlabel("time (ns)")
        #self.axes1.set_ylabel("density of events (events/ns)")
        self.axes1.set_ylabel("APD1")
        self.axes2.set_xlabel("time (ns)")
        #self.axes2.set_ylabel("density of events (events/ns)")
        self.axes2.set_ylabel("APD 2 and 3")
        self.axes3.set_xlabel("time (ns)")
        #self.axes3.set_ylabel("density of events (events/ns)")
        self.axes3.set_ylabel("2&3 conditioned\nto 1 in [t1a,t1b]")
        self.axes1.grid(True, color='gray')
        self.axes2.grid(True, color='gray')
        self.axes3.grid(True, color='gray')
        
        self.vline_t1a = self.axes1.axvline(self.t1a, color = 'b')
        self.vline_t1b = self.axes1.axvline(self.t1b, color = 'b')
        self.vline_t2a = self.axes2.axvline(self.t2a, color = 'r')
        self.vline_t2b = self.axes2.axvline(self.t2b, color = 'r')        

        # plot the data as a line series
        self.plot_data_APD_1 = self.axes1.plot(
                np.array(self.hist_x_APD_1),self.hist_APD_1,
                linewidth=1,
                color = self.color_dic["APD_1"]
                )[0]
        self.plot_data_APD_2 = self.axes2.plot(
                np.array(self.hist_x_APD_2),self.hist_APD_2,
                linewidth=1,
                color = self.color_dic["APD_2"]
                )[0]
        self.plot_data_APD_3 = self.axes2.plot(
                np.array(self.hist_x_APD_3),self.hist_APD_3,
                linewidth=1,
                color = self.color_dic["APD_3"]
                )[0]
        self.plot_data_APD_2_cond = self.axes3.plot(
                np.array(self.hist_x_APD_2_cond),self.hist_APD_2_cond,
                linewidth=1,
                color = self.color_dic["APD_2_cond"]
                )[0]
        self.plot_data_APD_3_cond = self.axes3.plot(
                np.array(self.hist_x_APD_3_cond),self.hist_APD_3_cond,
                linewidth=1,
                color = self.color_dic["APD_3_cond"]
                )[0]
        
        self.plot_data_vs_pulses = self.ax_vs_pulses.plot(
            np.array(self.x_vs_pulses),
            self.y_vs_pulses,
            linewidth=1,
            color=(1, 1, 0),
            )[0]  

    def set_data(self):
    #def set_data(self,choice):
        self.hist_APD_1 = self.hist_Y1
        self.hist_x_APD_1 = self.hist_X
        self.N_seg_APD_1=self.N_seg

        self.hist_APD_2 = self.hist_Y2
        self.hist_x_APD_2 = self.hist_X
        self.N_seg_APD_2=self.N_seg

        self.hist_APD_3 = self.hist_Y3
        self.hist_x_APD_3 = self.hist_X
        self.N_seg_APD_3=self.N_seg

        self.hist_APD_2_cond = self.hist_Y2_cond
        self.hist_x_APD_2_cond = self.hist_X
        self.N_seg_APD_2_cond=self.N_seg

        self.hist_APD_3_cond = self.hist_Y3_cond
        self.hist_x_APD_3_cond = self.hist_X
        self.N_seg_APD_3_cond=self.N_seg

    def draw_plot(self):
        """ Redraws the plot
        """
        self.plot_data_APD_1.set_xdata(self.hist_x_APD_1)
        if self.N_seg_APD_1!=0:
            self.plot_data_APD_1.set_ydata(self.hist_APD_1)
            
        self.plot_data_APD_3.set_xdata(self.hist_x_APD_3)
        if self.N_seg_APD_3!=0:
            self.plot_data_APD_3.set_ydata(self.hist_APD_3)
            
        self.plot_data_APD_2.set_xdata(self.hist_x_APD_2)
        if self.N_seg_APD_2!=0:
            self.plot_data_APD_2.set_ydata(self.hist_APD_2)
            
        self.plot_data_APD_2_cond.set_xdata(self.hist_x_APD_2_cond)
        if self.N_seg_APD_2_cond!=0:
            self.plot_data_APD_2_cond.set_ydata(self.hist_APD_2_cond)
            
        self.plot_data_APD_3_cond.set_xdata(self.hist_x_APD_3_cond)
        if self.N_seg_APD_3_cond!=0:
            self.plot_data_APD_3_cond.set_ydata(self.hist_APD_3_cond)

        #!#
        self.plot_data_vs_pulses.set_xdata(np.array(self.x_vs_pulses))
        if self.N_seg!=0: # if usefull here?
            self.plot_data_vs_pulses.set_ydata(self.y_vs_pulses)
        #!#
                
        self.axes1.relim()
        self.axes2.relim()
        self.axes3.relim()
        self.ax_vs_pulses.relim()
        self.axes1.autoscale_view()
        self.axes2.autoscale_view()
        self.axes3.autoscale_view()
        self.ax_vs_pulses.autoscale_view()
                   
        self.canvas.draw()
        self.canvas_vs_pulses.draw()

    #### GUI actions and cie
    def on_Run_button(self, event):        
        self.Run = self.run_button.GetValue()

        #choice = self.run_choice.GetSelection()
        
        if self.Run==True:
            wx.Yield()
            self.Run_exp()
            #self.Run_exp(choice)
        else:
            pass
    
    def on_save_plot(self,event):
        file_choices = "PNG (*.png)|*.png"
        
        dlg = wx.FileDialog(
            self, 
            message="Save plot as...",
            defaultDir=os.getcwd(),
            defaultFile="plot.png",
            wildcard=file_choices,
            style=wx.SAVE)
        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.canvas.print_figure(path, dpi=self.dpi)

    def on_save_dir(self,event):
        file_choices = "PNG (*.png)|*.png"
        
        dlg = wx.DirDialog(
            self, 
            message="Save in directory...",
            style=wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST)
        
        if dlg.ShowModal() == wx.ID_OK:
            self.save_dir = dlg.GetPath()
            print(self.save_dir)
            self.save_dir_bt.SetToolTip(wx.ToolTip(self.save_dir))

    def on_save_cfg(self,event):
        file_choices = "CFG (*.cfg)|*.cfg"
        
        dlg = wx.FileDialog(
            self, 
            message="Save cfg as...",
            defaultDir=os.getcwd(),
            defaultFile="defDLCZ.cfg",
            wildcard=file_choices,
            style=wx.SAVE)
        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.Save_config(path)

    def on_load_cfg(self,event):
        file_choices = "CFG (*.cfg)|*.cfg"
        
        dlg = wx.FileDialog(
            self, 
            message="Load cfg from...",
            defaultDir=os.getcwd(),
            defaultFile="*.cfg",
            wildcard=file_choices,
            style=wx.SAVE)
        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.Load_config(path)
       
    def on_exit(self, event):
        self.Run = False
        self.Save_config()
        pylab.close(self.fig) #otherwise the process is still running
        pylab.close(self.fig_vs_pulses) #otherwise the process is still running
        pylab.close(self.fig_coherence)
        self.Destroy()
        
    #### Some update functions. ####
    def update_results(self):
        self.text_n_APD_1.SetLabel("n1 (p1)= "+"%.5f" % self.n_APD_1)
        self.text_N_APD_1.SetLabel("("+str(self.N_APD_1)+"/"+str(self.N_seg_APD_1)+")")

        self.text_n_APD_2.SetLabel("n2 = "+"%.5f" % self.n_APD_2)
        self.text_N_APD_2.SetLabel("("+str(self.N_APD_2)+"/"+str(self.N_seg_APD_2)+")")

        self.text_n_APD_3.SetLabel("n3 = "+"%.5f" % self.n_APD_3)
        self.text_N_APD_3.SetLabel("("+str(self.N_APD_3)+"/"+str(self.N_seg_APD_3)+")")

        self.text_n_APD_2_cond.SetLabel("n2_cond = "+"%.5f" % self.n_APD_2_cond)
        self.text_N_APD_2_cond.SetLabel("("+str(self.N_APD_2_cond)+"/"+str(self.N_seg_APD_2_cond)+")")

        self.text_n_APD_3_cond.SetLabel("n3_cond = "+"%.5f" % self.n_APD_3_cond)
        self.text_N_APD_3_cond.SetLabel("("+str(self.N_APD_3_cond)+"/"+str(self.N_seg_APD_3_cond)+")")

        self.text_P1.SetLabel("P1 = "+"%.4f" % self.P1)
        self.text_P2.SetLabel("P2 = "+"%.4f" % self.P2)
        self.text_P12.SetLabel("P12 = "+"%.4f" % self.P12)
        self.text_PC.SetLabel("PC = "+"%.4f" % self.PC)
        
        self.text_G12.SetLabel("G12 = "+"%.4f" % self.G12)
        
        self.text_G2.SetLabel("G2 = "+"%.2f" % self.G2+"  N123 = %d" % self.N123)
        self.text_G2_noselect.SetLabel("G2n = "+"%.2f" % self.G2_noselect+"  N23 = %d" % self.N23)
        
    def update_time_param(self,event):
        self.t1a =self.spbutton_t1a.GetValue()
        self.t1b =self.spbutton_t1b.GetValue()
        self.t2a =self.spbutton_t2a.GetValue()
        self.t2b =self.spbutton_t2b.GetValue()
        self.period =self.spbutton_period.GetValue()
        self.vline_t1a.set_xdata(self.t1a)
        self.vline_t1b.set_xdata(self.t1b)
        self.vline_t2a.set_xdata(self.t2a)
        self.vline_t2b.set_xdata(self.t2b)        
        self.Save_config()

        self.set_results()
        self.update_results()
        self.draw_plot()       

    #### Some data analysis. ####
    def N_in_time(self,Hist,ta,tb):
        ''' Counts events in histogram between times ta and tb. '''                
        valid = np.where(Hist>ta,True,False) & np.where(Hist<tb,True,False)
        return sum(Hist[int(round(ta)):int(round(tb))])
    
    def set_results(self):
        ''' Compute the different interesting values from last data. '''
        # Counts are first selected with the windows defined by t1a,t1b or t2a, t2b
        self.N_APD_1   = self.N_in_time(self.hist_APD_1,  self.t1a/self.res,self.t1b/self.res)
        self.N_APD_2   = self.N_in_time(self.hist_APD_2,  self.t2a/self.res,self.t2b/self.res)
        self.N_APD_3  = self.N_in_time(self.hist_APD_3, self.t2a/self.res,self.t2b/self.res)
        self.N_APD_2_cond = self.N_in_time(self.hist_APD_2_cond,self.t2a/self.res,self.t2b/self.res)
        self.N_APD_3_cond = self.N_in_time(self.hist_APD_3_cond,self.t2a/self.res,self.t2b/self.res)

        # Number ofdetected photons per segment (single write/read cycle)
        if self.N_APD_1!=0:
            self.n_APD_1 = 1.0*self.N_APD_1/self.N_seg_APD_1
        else:
            self.n_APD_1=0
            
        if self.N_APD_2!=0:
            self.n_APD_2 = 1.0*self.N_APD_2/self.N_seg_APD_2
        else:
            self.n_APD_2=0
            
        if self.N_APD_3!=0:
            self.n_APD_3 = 1.0*self.N_APD_3/self.N_seg_APD_3
        else:
            self.n_APD_3=0
            
        if self.N_APD_2_cond!=0:
            self.n_APD_2_cond = 1.0*self.N_APD_2_cond/self.N_seg_APD_2_cond
        else:
            self.n_APD_2_cond=0

        if self.N_APD_3_cond!=0:
            self.n_APD_3_cond = 1.0*self.N_APD_3_cond/self.N_seg_APD_3_cond
        else:
            self.n_APD_3_cond=0


        self.P1 = self.n_APD_1
        self.P2 = self.n_APD_2 + self.n_APD_3

        self.P12 = self.n_APD_2_cond + self.n_APD_3_cond
                
        if self.n_APD_1>0:
            self.PC =  self.P12/(self.n_APD_1)
        else:
            self.PC = -1.0

            
        if self.P1>0 and self.P2>0:
            self.G12 = self.P12/(self.P1*self.P2)
        else:
            self.G12 = -1.0
    
        if self.n_APD_2_cond>0 and self.n_APD_3_cond>0:
            self.G2 = self.P1*(self.N123*1.0/self.N_seg_APD_1)/(self.n_APD_2_cond*self.n_APD_3_cond)
            #self.G2 = (self.N123*1.0/self.N_seg_APD_1)/(self.n_APD_2*self.n_APD_3)
        else:
            self.G2 = -1.0

        if self.n_APD_2>0 and self.n_APD_3>0:
            self.G2_noselect = (self.N23*1.0/self.N_seg_APD_1)/(self.n_APD_2*self.n_APD_3)
        else:
            self.G2_noselect = -1.0

    #### Acquisition and preprocessing/plotting loop. ####
    def Run_exp(self, cycles=-1,opt_dict=None,subdir=""):
    #def Run_exp(self, choice, cycles=-1,opt_dict=None,subdir=""):
        ''' Run the experiment. '''
        if not(self.Run):
            return
        
        #if (self.exp_ctrl and self.ctrl_exp_checkbx.GetValue()):
        #    self.chaudFroid.set_choice(self.sampleList[choice])

        # Create proper directories for data saving.
        start_date = datetime.today()
        run_dir = self.save_dir+start_date.strftime("\\20%y-%m-%d")
        if not os.path.exists(run_dir):
            os.makedirs(run_dir)
        run_dir2 = run_dir+subdir+start_date.strftime("\\%Hh%Mm%Ss")
        os.makedirs(run_dir2)
        print(run_dir2)

        # Depending on acquisition device, define:
        # self.res : time resolution, nanoseconds (typic 10ns)
        # self.period : is the time length of one segment (typic 1mus)
        # self.seg_size : is the number of points per segment
        if self.acq_mode=="lecroy":
            self.wdsc = self.scope.getwavedesc(1)

            fTreshold = 0.5
            threshold = (fTreshold+self.wdsc['vertical_offset'])/self.wdsc['vertical_gain']
        
            self.res = self.wdsc['horiz_interval']*1e9
          
        elif self.acq_mode=="sim":
            threshold = 5
            self.res = 10
       
        elif self.acq_mode=="fpga":
            self.res = 10
        
        res = self.res
        self.seg_size = int(round(self.period/self.res ))
        seg_size = self.seg_size

        # initialization of histograms data arrays
        self.hist_X = np.linspace(0,self.period-res,num=seg_size)
        self.hist_Y1 = (np.zeros((seg_size,), dtype=np.int))
        self.hist_Y2 = (np.zeros((seg_size,), dtype=np.int))
        self.hist_Y3 = (np.zeros((seg_size,), dtype=np.int))
        self.hist_Y2_cond = (np.zeros((seg_size,), dtype=np.int))
        self.hist_Y3_cond = (np.zeros((seg_size,), dtype=np.int))     

        self.N_seq = 0  # Number of segments (write/read cycle) per waveform (will be corrected is the following part)
        self.N_seg = 0  # Total number of segments acquired (=N_seq*number_of_waveforms_acquired)

        self.N123 = 0 #number of detected triple coincidences
        self.N23=0 #number of double coincidence on APD2 and APD3

        cnt=0
        
        ### loop  ###
        while self.Run: #and self.N < 5000:
            # Acquire data, wf1 is the waevform, wf1_tmp is the binary waveform.
            start = time.clock()
            start_total=start

            if self.acq_mode=='lecroy':
                wf1 = self.scope.getwaveform( 1, self.wdsc)
                wf1_tmp = np.where(wf1>threshold,1,0)

            elif self.acq_mode=='sim':
                t1=100
                t2=400
                t3=600
                wf1 = wfGen(self.res,t1,self.period,0)
                wf2 = wfGen(self.res,t2,self.period,2)
                wf3 = wfGen(self.res,t3,self.period,3)
                wf1_tmp = np.where(wf1>threshold,1,0)
                wf2_tmp = np.where(wf2>threshold,1,0)
                wf3_tmp = np.where(wf3>threshold,1,0)

            elif self.acq_mode=='fpga':
                self.Fpga_Acq.armed() # envoyer armed just aprs get Data pour aller plus vite
                self.Fpga_Acq.wait()
                wf1=self.Fpga_Acq.getData(self.Fpga_Acq.chan_dat_1)
                wf2=self.Fpga_Acq.getData(self.Fpga_Acq.chan_dat_2)
                wf3=self.Fpga_Acq.getData(self.Fpga_Acq.chan_dat_3)
                wf1_tmp=wf1
                wf2_tmp=wf2
                wf3_tmp=wf3
                
            else:
                print("ERROR: unknown acquisition mode")
                                
            #end = time.clock()
            #print "loading time = ", end - start

            #start = time.clock()
            N_seq = int(round(wf1.size/(self.period/res))) #Number of segments in a waveform
            self.N_seq=N_seq            
            self.N_seg += self.N_seq    #Total number of segments acquired
            
            if cnt==0: # Reinitialise x_vs_pulses/y_vs_pulses on first acquisition loop
                self.x_vs_pulses = np.linspace(1,N_seq,num=N_seq)
                self.y_vs_pulses = (np.zeros((N_seq,), dtype=np.int))

            # Detection of negative edges, trigg1 are the indexes when a negedge is detected

            try:
                trigg1 = np.where((wf1_tmp[0:N_seq*seg_size-1]-wf1_tmp[1:N_seq*seg_size])==1)[0]
                trigg2 = np.where((wf2_tmp[0:N_seq*seg_size-1]-wf2_tmp[1:N_seq*seg_size])==1)[0]
                trigg3 = np.where((wf3_tmp[0:N_seq*seg_size-1]-wf3_tmp[1:N_seq*seg_size])==1)[0]
            except ValueError as e:
                print("\n\nERROR\nValueError: The subsequence duration was probably not set correctly in the graphical interface. %s"%sys.exc_info()[0])

            # Eventually save raw data and negative edges detected
            if(self.save_wf_check.GetValue()):
                f1 = open(run_dir2+"\\wf1.npy", 'a+b')
                np.save(f1, wf1)
                f1.close()
                f2 = open(run_dir2+"\\wf2.npy", 'a+b')
                np.save(f2, wf2)
                f2.close()
                f3 = open(run_dir2+"\\wf3.npy", 'a+b')
                np.save(f3, wf3)
                f3.close()                
                
            if(self.save_trigger_check.GetValue()):
                T1 = open(run_dir2+"\\negedge1.npy", 'a+b')
                np.save(T1, trigg1)
                T1.close()
                T2 = open(run_dir2+"\\negedge2.npy", 'a+b')
                np.save(T2, trigg2)
                T2.close()
                T3 = open(run_dir2+"\\negedge3.npy", 'a+b')
                np.save(T3, trigg3)
                T3.close()
                        
            # list of corresponding segment number for each detected event
            trigg1_k = trigg1/seg_size # N.B. : division entiere
            trigg2_k = trigg2/seg_size # N.B. : division entiere
            trigg3_k = trigg3/seg_size # N.B. : division entiere
            # trigger time in the segment (read/write window)
            Trigg1 = trigg1%seg_size #Trigg1 is an array of indexes i where events were detected 0<=i<segment_length (write/read cycle length)
            Trigg2 = trigg2%seg_size #Trigg1 is an array of indexes i where events were detected 0<=i<segment_length (write/read cycle length)
            Trigg3 = trigg3%seg_size #Trigg1 is an array of indexes i where events were detected 0<=i<segment_length (write/read cycle length)
            
            self.y_vs_pulses[trigg1_k]=self.y_vs_pulses[trigg1_k]+1                      

            #end = time.clock()
            #print "processing time = ", end - start

            #array with "valid" events =in the right timing window defined by ta and tb
            valid_1 = np.where(np.where(Trigg1<self.t1b/res,True,False) & np.where(Trigg1>self.t1a/res,True,False))[0]
            vTrigg1_k=trigg1_k[valid_1] #indexes of the segments where this is a "valid" event
            valid_2 = np.where(np.where(Trigg2<self.t2b/res,True,False) & np.where(Trigg2>self.t2a/res,True,False))[0]
            vTrigg2_k=trigg2_k[valid_2]
            valid_3 = np.where(np.where(Trigg3<self.t2b/res,True,False) & np.where(Trigg3>self.t2a/res,True,False))[0]
            vTrigg3_k=trigg3_k[valid_3]

            # triple coincidence within the defined windows
            vTrigg123_k = set(set(vTrigg1_k).intersection(set(vTrigg2_k))).intersection(set(vTrigg3_k)) #indexes of the segments where there are 3 simultaneous "valid" events
            #vTrigg123_k = set(set(vTrigg3_k).intersection(set(vTrigg2_k)))
            vTrigg23_k = set(vTrigg2_k).intersection(set(vTrigg3_k))
            self.N123+=len(vTrigg123_k) #Counts the number of triple coincidences (with given timing windows)
            self.N23+=len(vTrigg23_k)

            #Detection events for APD2 and APD3 conditioned to a detection for APD1 in the timing window
            Trigg2_cond = Trigg2[np.where(np.in1d(trigg2_k,vTrigg1_k))[0]]
            Trigg3_cond = Trigg3[np.where(np.in1d(trigg3_k,vTrigg1_k))[0]]

            # Construct histograms
            hist1, bin_edge = np.histogram(Trigg1,bins = range(0,seg_size+1))
            hist1[0]=0
            hist2, bin_edge = np.histogram(Trigg2,bins = range(0,seg_size+1))
            hist2[0]=0
            hist3, bin_edge = np.histogram(Trigg3,bins = range(0,seg_size+1))
            hist3[0]=0
            hist2_cond, bin_edge = np.histogram(Trigg2_cond,bins = range(0,seg_size+1))
            hist2_cond[0]=0
            hist3_cond, bin_edge = np.histogram(Trigg3_cond,bins = range(0,seg_size+1))
            hist3_cond[0]=0

            self.hist_Y1 += hist1
            self.hist_Y2 += hist2
            self.hist_Y3 += hist3
            self.hist_Y2_cond += hist2_cond
            self.hist_Y3_cond += hist3_cond

            start = time.clock()

            # Update plot every 20 acquisition cycle.
            #if cnt%20==0:
            if cnt%5==0:
                self.set_data()
                #self.set_data(choice)   # sets the data in the right place depending on the choice
                self.set_results()      # basic analysis of data
                self.update_results()   # show results from analysis
                self.draw_plot()        # redraw

            cnt=cnt+1
            
            wx.Yield()
           
            #end = time.clock()
            #end_total = end

            if cnt==cycles:
                self.set_data()
                #self.set_data(choice)   # sets the data in the right place depending on the choice
                self.set_results()      # basic analysis of data
                self.update_results()   # show results from analysis
                self.draw_plot()        # redraw
                self.run_button.SetValue(False)
                self.Run=False

        # When stopped, eventually save report, and plots.    
        stop_date = datetime.today()
        if(self.save_report_check.GetValue()):
            #self.Write_report(start_date,stop_date,choice)
            self.Write_report(start_date,stop_date)
        if(self.save_plot_check.GetValue()):
            self.Save_plot(run_dir2)
        self.Save_config(run_dir2+"/config.cfg",opt_dict)

    #### Loading and saving routines.
    def Save_config(self,path="defDLCZ.cfg",opt_dict=None):
        #print "config saved\n"
        Config = ConfigParser.ConfigParser()
        Config.add_section('def')

        # time param
        Config.set('def','t1a', self.t1a)
        Config.set('def','t1b', self.t1b)
        Config.set('def','t2a', self.t2a)
        Config.set('def','t2b', self.t2b)
        Config.set('def','period', self.period)

        # save dir
        Config.set('def','save_dir', self.save_dir)

        # things to save
        Config.set('def','save_wf',self.save_wf_check.GetValue())
        Config.set('def','save_plot',self.save_plot_check.GetValue())
        Config.set('def','save_trigger',self.save_trigger_check.GetValue())
        Config.set('def','save_report',self.save_report_check.GetValue())

        #Config.set('def','auto_ctrl',self.ctrl_exp_checkbx.GetValue())
        #Config.set('def','cycles',self.spCtrl_cycles.GetValue())

        Config.set('def','comments',self.comments_ctrl.GetValue().encode('utf-8'))
        Config.set('def','N_seq',self.N_seq)

        if opt_dict != None:
            print(opt_dict)
            for k,v in opt_dict.items():
                print(k)
                print(v)
                Config.set('def',k,v)

        cfgfile = open(path,'w')
        Config.write(cfgfile)
        cfgfile.close()

    def Load_config(self,path="defDLCZ.cfg"):
        Config = ConfigParser.ConfigParser()
        Config.read(path)

        # time param
        self.t1a = Config.getfloat('def','t1a')
        self.t1b = Config.getfloat('def','t1b')
        self.t2a = Config.getfloat('def','t2a')
        self.t2b = Config.getfloat('def','t2b')
        self.period = Config.getfloat('def','period')
        #display
        self.spbutton_t1a.SetValue(self.t1a)
        self.spbutton_t1b.SetValue(self.t1b)
        self.spbutton_t2a.SetValue(self.t2a)
        self.spbutton_t2b.SetValue(self.t2b)
        self.spbutton_period.SetValue(self.period)
        self.update_time_param(self)

        #save dir
        self.save_dir = Config.get('def','save_dir')
        self.save_dir_bt.SetToolTip(wx.ToolTip(self.save_dir))

        # things to save
        self.save_wf_check.SetValue(Config.getboolean('def','save_wf'))
        self.save_plot_check.SetValue(Config.getboolean('def','save_plot'))
        self.save_trigger_check.SetValue(Config.getboolean('def','save_trigger'))
        self.save_report_check.SetValue(Config.getboolean('def','save_report'))
        if not self.save_trigger_check.GetValue():
            print("Warning, not saving raw data!!!")

        #self.ctrl_exp_checkbx.SetValue(Config.getboolean('def','auto_ctrl'))
        #self.spCtrl_cycles.SetValue(Config.getint('def','cycles'))
        
    #def Write_report(self,start_date,stop_date,choice):
    def Write_report(self,start_date,stop_date):
        #the_date = datetime.today()

        run_dir = self.save_dir+start_date.strftime("\\20%y-%m-%d")
        if not os.path.exists(run_dir):
            os.makedirs(run_dir)
        path = run_dir + '\\Nanof_DLCZ_report.txt'
        file = open(path,"a")
        file.write(start_date.strftime("\n\n%y-%m-%d // %H:%M:%S - "))
        file.write(stop_date.strftime("%H:%M:%S\n"))
        file.write("=========================\n")

        file.write("n_APD_1 = "+"%.5f" % self.n_APD_1)
        file.write("   ("+str(self.N_APD_1)+"/"+str(self.N_seg_APD_1)+")"+"\n")

        file.write("n_APD_2 = "+"%.5f" % self.n_APD_2)
        file.write("   ("+str(self.N_APD_2)+"/"+str(self.N_seg_APD_2)+")"+"\n")

        file.write("n_APD_3 = "+"%.5f" % self.n_APD_3)
        file.write("    ("+str(self.N_APD_3)+"/"+str(self.N_seg_APD_3)+")"+"\n")

        file.write("n_APD_2_cond = "+"%.5f" % self.n_APD_2_cond)
        file.write("    ("+str(self.N_APD_2_cond)+"/"+str(self.N_seg_APD_2_cond)+")"+"\n")

        file.write("n_APD_3_cond = "+"%.5f" % self.n_APD_3_cond)
        file.write("    ("+str(self.N_APD_3_cond)+"/"+str(self.N_seg_APD_3_cond)+")"+"\n")

        file.write("P1 = "+"%.5f" % self.P1+"\n")
        file.write("P2 = "+"%.5f" % self.P2+"\n")
        file.write("P12 = "+"%.5f" % self.P12+"\n")
        file.write("PC = "+"%.5f" % self.PC+"\n")
        file.write("G12 = "+"%.2f" % self.G12+"\n")
        file.write("G2 = "+"%.2f" % self.G2+"\n")
        file.write("G2n = "+"%.2f" % self.G2_noselect+"\n")

        file.write("\n\n" + self.comments_ctrl.GetValue().encode('utf-8'))

        file.write("\n--------------\n\n\n")

        file.close()

    def Save_plot(self,run_dir):
        self.canvas.print_figure(run_dir + "\plot.png", dpi=self.dpi)
        

if __name__ == '__main__':
    
    app = wx.App()
    #app.frame = DLCZ_nano_fiber(acq_mode='sim',exp_ctrl=False)
    app.frame = DLCZ_nano_fiber(acq_mode='fpga',exp_ctrl=False)
    app.frame.Show()
    app.MainLoop()
