#!/usr/bin/env python
try:
    import wx
except ImportError:
    raise ImportError,"The wxPython module is required to run this program"

import os
import pprint

import threading

numlist=['0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F',
       'G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V']
       
class SyncWidget(wx.ScrolledWindow):
    """ A graphical programmer for a Synchronizer"""
    def __init__(self,parent,dev):
        wx.ScrolledWindow.__init__(self,parent,wx.ID_ANY)
        self.SetScrollbars(0,1,0,1)
        self.dev=dev
        self.compileThread=None
        # process events from CompileThread
        EVT_COMPILED(self,self.onCompFinished)
        
        self.create_layout()
        
        self.dev.loadfromfile(os.path.join(os.path.dirname(__file__),'SyncConfigs','default.cfg'))
        self.getSyncCfg()
        self.getGUIcfg()

    def create_layout(self):                
        ### Subwidgets ###
        ## Control buttons
        self.savButton=wx.Button(self,-1,'Save Config')
        self.Bind(wx.EVT_BUTTON,self.onSavButton,self.savButton)
        self.savCButton=wx.Button(self,-1,'Save CompdFile')
        self.Bind(wx.EVT_BUTTON,self.onSavCButton,self.savCButton)
        self.loadFileButton=wx.Button(self,-1,'Load Cfg File')
        self.Bind(wx.EVT_BUTTON,self.onLoadFileButton,self.loadFileButton)        
        
        self.compButton=wx.Button(self,-1,'Compile')
        self.Bind(wx.EVT_BUTTON,self.onCompButton,self.compButton)
        self.load2FpgaButton=wx.Button(self,-1,'Load to FPGA')
        self.Bind(wx.EVT_BUTTON,self.onLoad2FpgaButton,self.load2FpgaButton)

        self.compandload2FpgaButton=wx.Button(self,-1,'Compile and L2FPGA')
        self.Bind(wx.EVT_BUTTON,self.onCompandload2FpgaButton,self.compandload2FpgaButton)

        self.startButton=wx.ToggleButton(self,-1,'Start')
        self.Bind(wx.EVT_TOGGLEBUTTON,self.onStartButton,self.startButton)

        self.contTrigButton=wx.ToggleButton(self,-1,'AutoTrig')
        self.Bind(wx.EVT_TOGGLEBUTTON,self.onContTrigButton,self.contTrigButton)

        self.updateTiButton=wx.Button(self,-1,'Update ti')
        self.Bind(wx.EVT_BUTTON,self.onUpdateTiButton,self.updateTiButton)

        # Buttons layout
        self.butSizer=wx.BoxSizer(wx.HORIZONTAL)
        # proportions: 1
        self.butSizer.Add(self.savButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.savCButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.loadFileButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.compButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.load2FpgaButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.compandload2FpgaButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.startButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.contTrigButton, proportion = 1, flag=wx.EXPAND)
        self.butSizer.Add(self.updateTiButton, proportion = 1, flag=wx.EXPAND)

        # Labels layout
        self.labelSizer=wx.BoxSizer(wx.VERTICAL)

        self.compiledLabel=wx.TextCtrl(self,-1,value='Compiled: %s'% str(self.dev.SyncCfg['compiled']))
        self.labelSizer.Add(self.compiledLabel, proportion = 1, flag=wx.EXPAND)

        self.loadedLabel=wx.TextCtrl(self,-1,value='Loaded to FPGA: %s'% str(self.dev.loaded))
        self.labelSizer.Add(self.loadedLabel, proportion = 1, flag=wx.EXPAND)

        self.butSizer.Add(self.labelSizer)
        
        ## List of Outputs and grid layout
        self.nOutputs=32#len(self.dev.SyncCfg['outputs'])
        self.nOutputsMax=self.nOutputs
        self.outputsLabel=wx.StaticText(self,-1,label=u'Outputs')
        self.nOutputsSpinbox=wx.SpinCtrl(self,-1,min=2,max=100,initial=self.nOutputs)
        self.Bind(wx.EVT_SPINCTRL,self.onNOutputsSpinbox,self.nOutputsSpinbox)
        self.periodLabel=wx.StaticText(self,-1,label=u'Period:')
        self.periodTxt=wx.TextCtrl(self,-1,value=str(self.dev.SyncCfg['period']))
        self.Bind(wx.EVT_TEXT,self.onPeriodChanged,self.periodTxt)

        self.numLabels=[]
        self.txtLabels=[]
        self.bncNameLabels=[]
        self.cfgTxt=[]
        self.enableManButs=[]
        self.onoffButs=[]
        self.subseqTxt=[]

        # Check boxes
        n_checkbox_preconfigs=20#8#4
        self.checkboxes=[[] for k in range(n_checkbox_preconfigs)]
        self.checklabels=[]
        self.checkbuttons=[]
        self.checkdescription=[]
        self.checkbutsSizer=wx.GridBagSizer()
        for k,e in enumerate(self.checkboxes):
            self.checklabels.append(wx.StaticText(self,-1,label=str(k+1)))
            self.checkbuttons.append(wx.ToggleButton(self,999-k,'Conf '+str(k+1)))
            self.Bind(wx.EVT_TOGGLEBUTTON,self.onCheckButton,self.checkbuttons[k])
            self.checkdescription.append(wx.TextCtrl(self,-1,value='Preconf '+str(k+1)))
            self.checkbutsSizer.Add(self.checkbuttons[k],(k%4,(k//4)*2),(1,1))#,wx.EXPAND)
            self.checkbutsSizer.Add(self.checkdescription[k],(k%4,(k//4)*2+1),(1,1),wx.EXPAND)

            ### Hide all those buttons and checkboxes that we don't really use for now.
            self.checkbuttons[k].Hide()
            self.checkdescription[k].Hide()
            for l in self.checklabels:
                l.Hide()
            ###
               
        self.gridSizer=wx.GridBagSizer()       
        self.gridSizer.Add(self.outputsLabel,(0,0),(1,3),wx.EXPAND)
        self.gridSizer.Add(self.nOutputsSpinbox,(0,3),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.periodLabel,(1,0),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.periodTxt,(1,3),(1,1),wx.EXPAND)
        for k,e in enumerate(self.checklabels):
            self.gridSizer.Add(e,(1,7+k),(1,1),wx.EXPAND)
                #(y,x),(dy,dx) position, span
        for i in range(self.nOutputs):
            self.addLine(i)
        self.gridSizer.AddGrowableCol(5,proportion=1)#third column can grow when resizing
        self.gridSizer.AddGrowableCol(6,proportion=1)      

        # Post defined timing values
        self.nt=len(self.dev.SyncCfg['tparams'])
        self.tLabels=[]
        self.tValues=[]
        self.tUserLabels=[]
        self.tSizer=wx.GridBagSizer()
        for i in range(self.nt):
            self.tLabels.append(wx.StaticText(self,-1,label='t'+numlist[i]))
            self.tUserLabels.append(wx.TextCtrl(self,300-i,value='',style=wx.TE_PROCESS_ENTER)) 
            self.tValues.append(wx.TextCtrl(self,i+300,value=str(self.dev.SyncCfg['tparams'][i]),style=wx.TE_PROCESS_ENTER))            
            #self.tSizer.Add(self.tLabels[i],(3*(i/8),i%8),(1,1),wx.EXPAND)
            #self.tSizer.Add(self.tUserLabels[i],(1+3*(i/8),i%8),(1,1),wx.EXPAND)
            #self.tSizer.Add(self.tValues[i],(2+3*(i/8),i%8),(1,1),wx.EXPAND)
            self.tSizer.Add(self.tLabels[i],(3*(i/16),i%16),(1,1),wx.EXPAND)
            self.tSizer.Add(self.tUserLabels[i],(1+3*(i/16),i%16),(1,1),wx.EXPAND)
            self.tSizer.Add(self.tValues[i],(2+3*(i/16),i%16),(1,1),wx.EXPAND)
            self.Bind(wx.EVT_TEXT,self.onTparamsChanged,self.tValues[i])
            self.Bind(wx.EVT_TEXT_ENTER,self.onUpdateTvalue,self.tValues[i])

            # Use a smaller MinSize for tValues TextCtrls
            self.tSizer.SetItemMinSize(self.tValues[i],(50,-1))
            self.tSizer.SetItemMinSize(self.tUserLabels[i],(50,-1))
            #if i<8:
            if i<16:
                self.tSizer.AddGrowableCol(i)

        # Subsequence parameters: tsub, nsubmax, Tsub
        self.tsubLabel=wx.StaticText(self,-1,label='Subseq starting time (musec):')
        self.nsubmaxLabel=wx.StaticText(self,-1,label='Number of subsequences:')
        self.TsubLabel=wx.StaticText(self,-1,label='Length of subseq (*10ns):')
        self.tsubText=wx.TextCtrl(self,-1,value='0')#str(self.dev.SyncCfg['subsequence']['tsub']))
        self.nsubmaxText=wx.TextCtrl(self,-1,value='0')#str(self.dev.SyncCfg['subsequence']['nsubmax']))
        self.TsubText=wx.TextCtrl(self,-1,value='0')#str(self.dev.SyncCfg['subsequence']['Tsub']))
        self.Bind(wx.EVT_TEXT,self.onSubSeq_main_Changed,self.tsubText)
        self.Bind(wx.EVT_TEXT,self.onSubSeq_main_Changed,self.nsubmaxText)
        self.Bind(wx.EVT_TEXT,self.onSubSeq_main_Changed,self.TsubText)
        
        self.subseqButton=wx.Button(self,-1,'Update Subseq')
        self.Bind(wx.EVT_BUTTON,self.onSubseqButton,self.subseqButton)

        self.subseqSizer=wx.GridBagSizer()
        self.subseqSizer.Add(self.tsubLabel,(0,0),(1,1),flag=wx.EXPAND)
        self.subseqSizer.Add(self.tsubText,(0,1),(1,1),flag=wx.EXPAND)
        self.subseqSizer.Add(self.nsubmaxLabel,(1,0),(1,1),flag=wx.EXPAND)
        self.subseqSizer.Add(self.nsubmaxText,(1,1),(1,1),flag=wx.EXPAND)
        self.subseqSizer.Add(self.TsubLabel,(2,0),(1,1),flag=wx.EXPAND)
        self.subseqSizer.Add(self.TsubText,(2,1),(1,1),flag=wx.EXPAND)
        self.subseqSizer.Add(self.subseqButton,(3,0),(1,2),flag=wx.EXPAND)
        
        ### Main layout
        self.sizer=wx.BoxSizer(wx.VERTICAL)
        self.horsizer=wx.BoxSizer(wx.HORIZONTAL)
        #self.horsizer2=wx.BoxSizer(wx.HORIZONTAL)

        #self.sizer.Add(self.horsizer2,proportion=8,flag=wx.EXPAND)
        #proportions: 8,8,1,1,1,1
        self.sizer.Add(self.gridSizer, proportion = 0, flag=wx.EXPAND)#ALIGN_TOP)#EXPAND)
        self.sizer.Add(self.horsizer,proportion=0,flag=wx.EXPAND)#ALIGN_TOP)#EXPAND)
        self.horsizer.Add(self.checkbutsSizer, proportion=0, flag=wx.EXPAND)#ALIGN_TOP)#EXPAND)
        self.sizer.Add(self.tSizer, proportion=0, flag=wx.EXPAND)#ALIGN_TOP)#EXPAND)
        self.horsizer.Add(self.subseqSizer, proportion=0, flag=wx.EXPAND)#ALIGN_TOP)#EXPAND)
        self.sizer.Add(self.butSizer, proportion = 0, flag=wx.EXPAND)#ALIGN_TOP)#EXPAND)
        
        self.SetSizer(self.sizer)
    
    # Buttons events
    def onSavButton(self,event):
        dlg = wx.FileDialog(
            self, message="Save file as...",
            defaultDir=os.path.dirname(__file__)+'/SyncConfigs',
            defaultFile="",
            wildcard=   'Config files (*.cfg)|*.cfg|' \
                        'Config and bit files (*.cfg,*.bit,*.xsvf)|*.cfg;*.bit;*.xsvf|' \
                        'All files (*)|*',
            style= wx.FD_SAVE
            )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.dev.savetofile(path)
            self.writeGUIcfg(path)

    def writeGUIcfg(self,path):
        self.GUIcfg={}
        n_check=len(self.checkbuttons)
        self.GUIcfg['checkboxes_values']=[[self.checkboxes[k][i].GetValue() for i in range(self.nOutputs)] for k in range(n_check)]
        self.GUIcfg['checkdescriptions']=[self.checkdescription[k].GetValue() for k in range(n_check)]
        self.GUIcfg['tUserLabels']=[self.tUserLabels[i].GetValue() for i in range(self.nt)]       
        f=open(path,'a')
        f.write( 'self.GUIcfg = ' + pprint.pformat(self.GUIcfg) + '\n' )
        #f.write( 'self.GUIcfg = ' + repr(self.GUIcfg) + '\n' )
        f.close()

    def getGUIcfg(self):
        self.GUIcfg=self.dev.GUIcfg
        for k,l in enumerate(self.GUIcfg['checkboxes_values']):
            for i,v in enumerate(l):
                if i < len(l):
                    self.checkboxes[k][i].SetValue(v)
        for k,s in enumerate(self.GUIcfg['checkdescriptions']):
            self.checkdescription[k].SetValue(s)
        for i,s in enumerate(self.GUIcfg['tUserLabels']):
            self.tUserLabels[i].SetValue(s)

    def onSavCButton(self,event):
        dlg = wx.FileDialog(
            self, message="Save file as...",
            defaultDir=os.path.dirname(__file__)+'/SyncConfigs',
            defaultFile="",
            wildcard=   'Config files (*.cfg)|*.cfg|' \
                        'Config and bit files (*.cfg,*.bit,*.xsvf)|*.cfg;*.bit;*.xsvf|' \
                        'All files (*)|*',
            style= wx.FD_SAVE
            )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.dev.saveCompdFile(path)

    def onLoadFileButton(self,event):
        dlg = wx.FileDialog(
            self, message="Choose a .cfg file to load",
            defaultDir=os.path.dirname(__file__)+'/SyncConfigs',
            defaultFile="default.cfg",
            wildcard=   'Config files (*.cfg)|*.cfg|' \
                        'Config and bit files (*.cfg,*.bit,*.xsvf)|*.cfg;*.bit;*.xsvf|' \
                        'All files (*)|*',
            style=wx.FD_OPEN
            )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            if path.endswith('.cfg'):
                self.dev.loadfromfile(path)
            else:
                print('Wrong file type, a .cfg file should be chosen.')
            self.loadedLabel.SetValue("Not loaded to FPGA")
            self.getSyncCfg()
            self.getGUIcfg()

    def getSyncCfg(self):
        cfg=self.dev.SyncCfg
        compd=cfg['compiled']
        
        self.nOutputsSpinbox.SetValue(len(cfg['outputs']))
        self.onNOutputsSpinbox(None)

        self.periodTxt.SetValue(str(cfg['period']).replace(' ','').replace("'","").replace('"',''))
        for i in range(len(cfg['outputs'])):
            self.bncNameLabels[i].SetValue(cfg['outputs'][i]['bnc_name'])
            self.txtLabels[i].SetValue(cfg['outputs'][i]['description'])
            self.cfgTxt[i].SetValue(str(cfg['outputs'][i]['sequence'])[1:-1].replace(' ','').replace("'","").replace('"',''))
            self.enableManButs[i].SetValue(cfg['outputs'][i]['manual_enable'])
            self.onoffButs[i].SetValue(cfg['outputs'][i]['manual_value'])
            try:
                self.subseqTxt[i].SetValue(str(cfg['outputs'][i]['subseq'])[1:-1].replace(' ','').replace("'","").replace('"',''))
            except KeyError:
                print('Warning: Probably no subsequence defined in this config.')
        for i in range(len(cfg['tparams'])):#self.nt):
            self.tValues[i].SetValue(str(cfg['tparams'][i]))
        try:
            subseq=cfg['subsequence']
            self.tsubText.SetValue(str(subseq['tsub']))
            self.nsubmaxText.SetValue(str(subseq['nsubmax']))
            self.TsubText.SetValue(str(subseq['Tsub']))
        except KeyError:
            print('Warning: Probably no subsequence defined in this config.')
        
        self.contTrigButton.SetValue(cfg['autotrig'])
        self.compiledLabel.SetValue('Compiled: '+str(compd))
        cfg['compiled']=compd



    '''
    # No threading version
    def onCompButton(self,event):
        self.dev.compCfg(nOutputs=self.nOutputs)
        self.compiledLabel.SetValue("Compiled")
        self.loadedLabel.SetValue("Not loaded to FPGA")
    '''
    def onCompButton(self,event):
        if not self.compileThread:
            self.compileThread=CompileThread(self)
            self.compiledLabel.SetValue("Compiling...")
        else:
            self.compiledLabel.SetValue("Already compiling...")
            self.compileThread=False

    def onCompFinished(self,event):
        self.compiledLabel.SetValue("Compiled")
        self.loadedLabel.SetValue("Not loaded to FPGA")
        self.compileThread=None

    def onLoad2FpgaButton(self,event):
        self.dev.loadCfg()
        self.loadedLabel.SetValue("Loaded to FPGA")
        self.startButton.SetValue(False)

    def onCompandload2FpgaButton(self,event):
        self.onCompButton(event)
        self.onLoad2FpgaButton(event)

    def onStartButton(self,event):
        if self.startButton.GetValue():
            self.dev.start()
        else:
            self.dev.stop()

    # Parsing Utils
    def intIfDigit(self,x):
        if x.isdigit():
            return int(x)
        else:
            return str(x)

    def parseSequence(self,i):
        line=self.cfgTxt[i]
        s=line.GetValue()
        if s=='':
            return []
        splittedOnce=s[1:len(s)-1].split('),(')
        splittedTwice=[]
        for elem in splittedOnce:
            l=elem.split(',')
            l2=map(self.intIfDigit,l)
            t=tuple(l2)
            splittedTwice.append(t)
        return splittedTwice

    def parseSubSeq(self,i):
        line=self.subseqTxt[i]
        s=line.GetValue()
        if s=='':
            return []
        splittedOnce=s[1:len(s)-1].split('),(')
        splittedTwice=[]
        for elem in splittedOnce:
            l=elem.split(',')
            l2=map(self.intIfDigit,l)
            t=tuple(l2)
            splittedTwice.append(t)
        return splittedTwice
        
    def parsePeriod(self):
        periodtxt=self.periodTxt.GetValue()
        l=periodtxt[1:len(periodtxt)-1].split(',')
        return tuple(map(self.intIfDigit,l))  

    # On changed events
    def notCompiled(self):
        self.dev.SyncCfg['compiled']=False
        self.compiledLabel.SetValue("Not compiled")
    
    def onBncnameChanged(self,event):
        i = event.GetId()
        self.dev.SyncCfg['outputs'][i]['bnc_name']=self.bncNameLabels[i].GetValue().upper().replace(' ','')
        self.notCompiled()
        
    def onSequenceChanged(self,event):
        i = event.GetId()-200
        self.dev.SyncCfg['outputs'][i]['sequence']=self.parseSequence(i)
        self.notCompiled()

    def onSubSeqChanged(self,event):
        i = event.GetId()-600
        self.dev.SyncCfg['outputs'][i]['subseq']=self.parseSubSeq(i)
        self.notCompiled()

    def onSubSeq_main_Changed(self,event):
        d={}
        d['tsub']=int(self.tsubText.GetValue())
        d['nsubmax']=int(self.nsubmaxText.GetValue())
        d['Tsub']=int(self.TsubText.GetValue())
        self.dev.SyncCfg['subsequence']=d

    def onSubseqButton(self,event):
        self.dev.updateSubsequence()

    def onPeriodChanged(self,event):
        self.dev.SyncCfg['period']=self.parsePeriod()
        self.notCompiled()
        
    def onOutputNameChanged(self,event):
        i = event.GetId()-100
        self.dev.SyncCfg['outputs'][i]['description']=self.txtLabels[i].GetValue()

    def onNOutputsSpinbox(self,event):
        ''' Dynamically add or remove line. '''
        newValue = self.nOutputsSpinbox.GetValue()
        
        if newValue < self.nOutputs:
            for i in range(newValue,self.nOutputs):
                self.hideLine(i)

        elif newValue < self.nOutputsMax:
            for i in range(self.nOutputs,newValue):
                self.showLine(i)
                
        else:
            for i in range(self.nOutputs,self.nOutputsMax):
                self.showLine(i)            
            for i in range(self.nOutputsMax,newValue):
                self.addLine(i)          
            self.nOutputsMax=newValue

        self.nOutputs=newValue
        self.Layout()#forces refreshment on the panel layout

    def onEnableManButs(self,event):
        i = event.GetId()-400
        self.dev.SyncCfg['outputs'][i]['manual_enable']=int(self.enableManButs[i].GetValue())
        self.dev.updateManualEnable()

    def onOnoffButs(self,event):
        i = event.GetId()-500
        self.dev.SyncCfg['outputs'][i]['manual_value']=int(self.onoffButs[i].GetValue())
        self.dev.updateManualValues()

    def onContTrigButton(self, event):
        self.dev.SyncCfg['autotrig']=self.contTrigButton.GetValue()
        self.dev.updateAutotrig()

    def onTparamsChanged(self,event):
        i = event.GetId()-300
        val = self.tValues[i].GetValue()
        if val == '':
            val = 0
        while len(self.dev.SyncCfg['tparams'])<i+1:
            self.dev.SyncCfg['tparams'].append(0)
        self.dev.SyncCfg['tparams'][i]=int(val)

    def onUpdateTvalue(self,event):
        i = event.GetId()-300
        self.dev.updateTparam(i)        

    def onUpdateTiButton(self,event):
        self.dev.updateTparams()

    def onCheckButton(self,event):
        k=999-event.GetId()
        n=len(self.checkbuttons)
        others=range(n)
        others.remove(k)
        if self.checkbuttons[k].GetValue():
            for o in others:
                self.checkbuttons[o].SetValue(False)
            for i in range(self.nOutputs):
                newvalue=self.checkboxes[k][i].GetValue()
                self.dev.SyncCfg['outputs'][i]['manual_enable']=int(newvalue)
                self.enableManButs[i].SetValue(newvalue)
            self.dev.updateManualEnable()
        
    ### Layout utils
    def addLine(self,i):
        self.numLabels.append(wx.StaticText(self,-1,label=str(i+1)))
        self.bncNameLabels.append(wx.TextCtrl(self,i,value='DIO '+str(i)))
        self.txtLabels.append(wx.TextCtrl(self,i+100,value='Output '+str(i+1)))
        self.cfgTxt.append(wx.TextCtrl(self,i+200,value='(0,0,0,0)'))
        self.enableManButs.append(wx.ToggleButton(self,i+400,'Manual'))
        self.onoffButs.append(wx.ToggleButton(self,i+500,'High'))
        self.subseqTxt.append(wx.TextCtrl(self,i+600,value=''))

        for preconfig in self.checkboxes:
            preconfig.append(wx.CheckBox(self,-1))

        self.gridSizer.Add(self.numLabels[i],(i+2,0),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.bncNameLabels[i],(i+2,1),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.txtLabels[i],(i+2,2),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.cfgTxt[i],(i+2,5),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.enableManButs[i],(i+2,3),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.onoffButs[i],(i+2,4),(1,1),wx.EXPAND)
        self.gridSizer.Add(self.subseqTxt[i],(i+2,6),(1,1),wx.EXPAND)

        for k,preconfig in enumerate(self.checkboxes):
            self.gridSizer.Add(preconfig[i],(i+2,7+k),(1,1),wx.EXPAND)

        self.Bind(wx.EVT_TEXT,self.onBncnameChanged,self.bncNameLabels[i])
        self.Bind(wx.EVT_TEXT,self.onSequenceChanged,self.cfgTxt[i])
        self.Bind(wx.EVT_TEXT,self.onOutputNameChanged,self.txtLabels[i])
        self.Bind(wx.EVT_TOGGLEBUTTON,self.onEnableManButs,self.enableManButs[i])
        self.Bind(wx.EVT_TOGGLEBUTTON,self.onOnoffButs,self.onoffButs[i])
        self.Bind(wx.EVT_TEXT,self.onSubSeqChanged,self.subseqTxt[i])
        if i >= len(self.dev.SyncCfg['outputs']):
            self.dev.SyncCfg['outputs'].append({'description':'Output '+str(i+1), 'bnc_name':'DIO '+str(i), 'sequence':[(0,0,0,0)], 'manual_enable':0, 'manual_value':0,'subseq':[]})

        for preconfig in self.checkboxes:     
            preconfig[i].Hide()        #Hide these checkboxes that we don't really use.
    
    def hideLine(self,i):
        self.numLabels[i].Hide()
        self.bncNameLabels[i].Hide()
        self.txtLabels[i].Hide()
        self.cfgTxt[i].Hide()
        self.enableManButs[i].Hide()
        self.onoffButs[i].Hide()
        self.subseqTxt[i].Hide()

        for preconfig in self.checkboxes:     
            preconfig[i].Hide()
        
    def showLine(self,i):
        self.numLabels[i].Show()
        self.bncNameLabels[i].Show()
        self.txtLabels[i].Show()
        self.cfgTxt[i].Show()
        self.enableManButs[i].Show()
        self.onoffButs[i].Show()
        self.subseqTxt[i].Show()
        
        for preconfig in self.checkboxes:     
            preconfig[i].Show()

        if i >= len(self.dev.SyncCfg['outputs']):
            self.dev.SyncCfg['outputs'].append({'description':'Output '+str(i+1), 'bnc_name':'DIO '+str(i), 'sequence':[(0,0,0,0)], 'manual_enable':0, 'manual_value':0,'subseq':[]})


class TestDev:
    def __init__(self):
        self.SyncCfg={
            'period':(1,0,0,0),
            'outputs':[
                {'description':'First output', 'bnc_name':'LED0', 'sequence':[(0,0,0,0)],'subseq':[(0,0,0,0)], 'manual_enable':1, 'manual_value':0},
                {'description':'Second output', 'bnc_name':'LED1', 'sequence':[(0,0,0,0)],'subseq':[], 'manual_enable':0, 'manual_value':1},
                {'description':'Third output', 'bnc_name':'LED2', 'sequence':[(0,0,0,0)], 'subseq':[],'manual_enable':1, 'manual_value':1},
                {'description':'Fourth output', 'bnc_name':'LED3', 'sequence':[(0,0,0,0)], 'subseq':[],'manual_enable':0, 'manual_value':0},
       
                ],
            'tparams':[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
            'subsequence':{'tsub':53000,'nsubmax':100,'Tsub':1000},
            'compiled':False,
            'autotrig':True,
            'filepath':'',
            }
        self.loaded=False

    def start(self):
        print 'Starting'

    def stop(self):
        print 'Stopping'

    def compCfg(self,nOutputs=8):
        print('Compiling')

    def loadCfg(self):
        print('Loading')

    def updateparams(self):
        print('Communicating with Fpga')

    def updateManualEnable(self):
        print('Communicating with Fpga')

    def updateManualValues(self):
        print('Communicating with Fpga')

    def updateAutotrig(self):
        print('Communicating with Fpga')

    def updateTparams(self):
        print('Communicating with Fpga')

    def savetofile(self,path):
        print ('Saving to '+path)

    def loadfromfile(self,path):
        exec(open(path,'r').read())
        self.loaded=False
        print ('Loading from '+path)

    def updateSubsequence(self):
        print ('Updating subsequence')
        print (self.SyncCfg['subsequence'])
        
    def saveCompdFile(self,filename):
        print ("TestDev: saving %s."%filename)

class CompileThread(threading.Thread):
    def __init__(self,parent):
        threading.Thread.__init__(self)
        self.parent=parent
        self.start()

    def run(self):
        self.parent.dev.compCfg(nOutputs=self.parent.nOutputs)
        wx.PostEvent(self.parent, CompiledEvent())

EVT_COMPILED_ID= wx.NewId()

def EVT_COMPILED(win,func):
    win.Connect(-1,-1,EVT_COMPILED_ID, func)

class CompiledEvent(wx.PyEvent):
    def __init__(self):
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_COMPILED_ID)

if __name__=="__main__":
    test=False

    app = wx.App(False)

    if test:
        device=TestDev()
        frame = wx.Frame(parent=None,id=-1,title="TestFrame",size=wx.Size(800,500))        
    else:
        import synchronizer
        device=synchronizer.Synchronizer()   
        frame = wx.Frame(parent=None,id=-1,title="FPGA Synchronizer",size=wx.Size(800,500))

    frame.panel= SyncWidget(parent=frame,dev=device)
    app.frame = frame    
    app.frame.Show()
    app.MainLoop()
    

