#!/usr/bin/env python
"""
Defines two functions write_verilog and write_ucf.
    write_verilog takes a timing sequence array as input and writes a coresponding verilog file as output.
    write_ucf takes bnc names input and writes a coresponding ucf file as output.

Uses files in simplesync_files/, verilog code from intro.v_part and end.v_part,
writes in sync.v
"""
import main

# Fpga frequency.
freq = 100*10**6

# Path management for files to read/write
# if the module is not imported __file__ is not defined, or dirname(__file__)='', otherwise dirname(__file__)='/path/to/dir'
import os.path
if '__file__' in locals():
    base=os.path.dirname(__file__)+'/'
else:
    base=''
base+='simplesync_files'

def sync_files():
    return base

def write_verilog(outputs):
    outString=""
    for i, output in enumerate(outputs):
        outString+="\n\t\t // output %d" % i
        for date in output:
            periods = (date[0]+date[1]*10**(-3)+date[2]*10**(-6)+date[3]*10**(-9))*freq
            outString += """
                        if(counter == %d)           
                        begin
                            out[%d]=out[%d]+1;
                        end
                        """% (periods, i, i)

    intro = open(base+"/intro.v_part","r")
    end = open(base+"/end.v_part","r")
    verilogString = intro.read() + outString + end.read()

    intro.close()
    end.close()

    verilogfile = open(base+"/sync.v", "w")
    verilogfile.write(verilogString)
    verilogfile.close()
    
def write_ucf(ucfcfg):
    ''' Adapts the .ucf constraint file to the list ucfcfg of bnc outputs. '''
    outString=""
    for i,bnc in enumerate(ucfcfg):
        outString+='NET "outputs<%d>"         LOC = "%s" | IOSTANDARD = "LVCMOS33";\n'%(i,main.bnc2ucf[bnc])

    intro = open(base+"/intro.ucf_part","r")
    end = open(base+"/end.ucf_part","r")
    verilogString = intro.read() + outString + end.read()

    intro.close()
    end.close()

    ucffile = open(base+"/sync.ucf", "w")
    ucffile.write(verilogString)
    ucffile.close()

# Test du script
if __name__ =="__main__":
    outputs = [# Format: (secs, millisecs, microsecs, nanosecs)
    # output 0, here output 0 is used to define the period of the sequence.
    [(8,0,0,0)],
    # output 1
    [(1,0,0,0),(2,0,0,0)],
    # output 2
    [(2,0,0,0),(3,0,0,0)],
    # output 3
    [(3,0,0,0),(4,0,0,0)],
    # output 4
    [(4,0,0,0),(5,0,0,0)],
    # output 5
    [(5,0,0,0),(6,0,0,0)],
    # output 6
    [(6,0,0,0),(7,0,0,0)],
    # output 7
    [(7,0,0,0),(8,0,0,0)],
    ]
    write_verilog(outputs)

    l=[u'LED0', u'LED1', u'LED2', u'LED3', u'LED4', u'LED5', u'LED6', u'LED7']
    write_ucf(l)

    main.veri2bit('sync',base)
    main.bit2Fpga('simplesync_files/sync.xsvf')
    
